package com.gmh.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gmh.entity.*;
import com.gmh.entity.properties.FileServerProperties;
import com.gmh.entity.query.ExecuteTemplateQuery;
import com.gmh.entity.vo.ReadNameVo;
import com.gmh.service.FileService;
import com.gmh.service.GmhFileLogService;
import com.gmh.service.SysBaijiaxingService;
import com.gmh.utils.POIExcelUtil;
import com.gmh.utils.StringUtils;
import lombok.AllArgsConstructor;
import lombok.Getter;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/excel")
public class ExcelController {

    @Autowired
    private FileService fileService;

    @Autowired
    private GmhFileLogService gmhFileLogService;

    @Autowired
    private FileServerProperties fileProperties;

    @Autowired
    private SysBaijiaxingService baijiaxingService;

    @Autowired
    private HttpServletRequest request;

    /**
     * HashMap的Key为SessionId
     *
     * 通过SessionId为每一个会话保存自己的对象副本
     *
     * 不适用于集群环境
     */
    private final Map<String, ExcelInfo> excelInfoMap = new HashMap<>();

    private static final String KEY_XINGMING = "姓名";

    @RequestMapping("/t01")
    public R test01() {
        return R.ok("ok");
    }

    @RequestMapping("/dataSourceImport")
    public R dataSourceImport(MultipartFile file, String dataSourceImportModel) throws IOException {
        excelInfoMap.put(request.getSession().getId(), new ExcelInfo());
        DataSourceImportModel importModel = DataSourceImportModel.findEnum(dataSourceImportModel);
        Workbook workbook = POIExcelUtil.readExcelFromInputStream(file.getInputStream(), file.getOriginalFilename());
        FileEntity fileEntity = fileService.uploadFile(file);
        gmhFileLogService.recordLog(fileEntity, GmhFileLog.OperateType.D_S_IMP);
        if (DataSourceImportModel.AUTO_NAME.equals(importModel)) {
            List<ReadNameVo> readNameVoList = new ArrayList<>();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                ReadNameVo readNameVo = new ReadNameVo();
                Sheet sheet = workbook.getSheetAt(i);
                List<Map<String, String>> excelMaps = POIExcelUtil.toListMap(sheet);
                if (excelMaps.isEmpty()) {
                    continue;
                }
                Map<String, String> firstElement = excelMaps.get(0);
                String xmKey = KEY_XINGMING;
                for (String key : firstElement.keySet()) {
                    if (KEY_XINGMING.equals(firstElement.get(key))) {
                        xmKey = key;
                    }
                }
                if (firstElement.containsKey(xmKey)) {
                    if (!KEY_XINGMING.equals(xmKey)) {
                        excelMaps.remove(firstElement);
                    }
                    readNameVo = getReadNameDataForNameKey(excelMaps, xmKey);
                } else {
                    Map<String, Integer> map = new HashMap<>();
                    for (Map<String, String> excelMap : excelMaps) {
                        for (String key : excelMap.keySet()) {
                            String value = excelMap.get(key);
                            if (baijiaxingService.likeChineseName(value)) {
                                Integer count = map.getOrDefault(key, 0);
                                map.put(key, ++count);
                            }
                        }
                    }
                    List<Map.Entry<String,Integer>> list = new ArrayList<>(map.entrySet());
//                list.sort(Comparator.comparingInt(Map.Entry::getValue));        //升序
                    list.sort((o1, o2) -> (o2.getValue() - o1.getValue()));
                    String key = list.get(0).getKey();
                    readNameVo = getReadNameDataForNameKey(excelMaps, key);
                }
                readNameVo.setSheetName(sheet.getSheetName());
                readNameVoList.add(readNameVo);
            }
            ExcelInfo excelInfo = excelInfoMap.get(request.getSession().getId());
            excelInfo.setReadNameVoList(readNameVoList);
            return R.ok().setData(readNameVoList);
        }

        return R.error();

    }

    public ReadNameVo getReadNameDataForNameKey(List<Map<String, String>> excelMaps, final String key) {
        ReadNameVo readNameVo = new ReadNameVo();
        // 去重前
        List<GmhUser> oldNameList = excelMaps.stream()
                .map(map -> new GmhUser(map.get(key)))
                .filter(item -> StringUtils.isNotBlank(item.getName()))
                .collect(Collectors.toList());
        // 去重后
        LinkedHashSet<GmhUser> newNameList = new LinkedHashSet<>(oldNameList);
        readNameVo.setNameList(newNameList);
        // 重复检测
        int repeatCount = oldNameList.size() - newNameList.size();
        if (repeatCount > 0) {
            readNameVo.setHasRepeat(true);
            readNameVo.setRepeatCount(repeatCount);
        } else {
            readNameVo.setHasRepeat(false);
            readNameVo.setRepeatCount(0);
        }
        return readNameVo;
    }

    /**
     * 模板导入
     * @param file excel文件
     * @param templateSheetNum 模板所在Sheet页
     * @return
     * @throws IOException
     */
    @RequestMapping("/template")
    public R templateUpload(MultipartFile file, String templateSheetNum) throws IOException {
        Workbook workbook = POIExcelUtil.readExcelFromInputStream(file.getInputStream(), file.getOriginalFilename());
        FileEntity fileEntity = fileService.uploadFile(file);
        gmhFileLogService.recordLog(fileEntity, GmhFileLog.OperateType.TEMPL_IMP);
        ExcelInfo excelInfo = excelInfoMap.get(request.getSession().getId());
        excelInfo.setTemplateOriginalFilename(file.getOriginalFilename());
        excelInfo.setTemplateSheetNum(Integer.parseInt(templateSheetNum) - 1);
        Sheet sheet = workbook.getSheetAt(excelInfo.getTemplateSheetNum());
        if (sheet == null) {
            throw new RuntimeException("sheet页不存在");
        }
        excelInfo.setTemplateWorkbook(workbook);
        return R.ok();
    }

    @RequestMapping("/cellNumSearch")
    public R analysisCell(@Validated RowCellNum rowCellNum) {
        ExcelInfo excelInfo = excelInfoMap.get(request.getSession().getId());
        if (excelInfo.getTemplateWorkbook() == null) {
            return R.error("模板文件出问题了，请重新上传");
        }
        Sheet sheet = excelInfo.getTemplateWorkbook().getSheetAt(excelInfo.getTemplateSheetNum());
        rowCellNum = POIExcelUtil.analysisCellRowNum(rowCellNum);
        Row row = sheet.getRow(rowCellNum.getRowNum());
        if (row == null) {
            return R.ok().setData("");
        }
        Cell cell = row.getCell(rowCellNum.getCellNum());
        if (cell == null) {
            return R.ok().setData("");
        }
        String stringCellVal = POIExcelUtil.getStringCellVal(cell);
        if (StrUtil.isEmpty(stringCellVal)) {
            return R.ok().setData("");
        }
        return R.ok().setData(stringCellVal);
    }


    @PostMapping("/executeTemplate")
    public R executeTemplate(@RequestBody ExecuteTemplateQuery executeTemplateQuery) throws IOException {
        // TODO 有表达式和无表达式兼容处理
        String newCellData = executeTemplateQuery.getNewCellData();
        String expr = newCellData.substring(newCellData.indexOf("@{"), newCellData.indexOf("}") + 1);
        String exprVal = newCellData.substring(newCellData.indexOf("@{") + 2, newCellData.indexOf("}"));
        String[] exprArr = exprVal.split("\\.");
        String sheetName = exprArr[0];
        String field = exprArr[1];

        ExcelInfo excelInfo = excelInfoMap.get(request.getSession().getId());
        Optional<ReadNameVo> first = excelInfo.getReadNameVoList()
                .stream()
                .filter(item -> item.getSheetName().equalsIgnoreCase(sheetName))
                .findFirst();
        if (!first.isPresent()) {
            throw new RuntimeException("未找到sheet页，可能是表达式错误");
        }
        ReadNameVo readNameVo = first.get();
        POIExcelUtil.removeModelSheet(excelInfo.getTemplateWorkbook(), excelInfo.getTemplateSheetNum());
        POIExcelUtil.batchCloneSheet(excelInfo.getTemplateWorkbook(), excelInfo.getTemplateSheetNum(), readNameVo.getNameList().size());
        ArrayList<GmhUser> gmhUsers = new ArrayList<>(readNameVo.getNameList());
        RowCellNum rowCellNum = POIExcelUtil.analysisCellRowNum(executeTemplateQuery.getCellRowStr());
        for (int i = 0; i < excelInfo.getTemplateWorkbook().getNumberOfSheets(); i++) {
            Sheet sheet = excelInfo.getTemplateWorkbook().getSheetAt(i);
            if (sheet == null) {
                continue;
            }
            Row row = sheet.getRow(rowCellNum.getRowNum());
            Cell cell = row.getCell(rowCellNum.getCellNum());
            cell.setCellValue(newCellData.replace(expr, StringUtils.nameFormat(gmhUsers.get(i).getName())));
        }
        // TODO 文件保存到本地，文件信息入库
        String localPath = fileProperties.getLocal().getPath();
        File parentFile = new File(localPath);
        if (!parentFile.exists()) {
            if (!parentFile.mkdirs()) {
                throw new RuntimeException("创建保存路径失败");
            }
        }
        FileEntity fileEntity = new FileEntity();
        String extName = FileUtil.extName(excelInfo.getTemplateOriginalFilename());
        String originalFilename;
        if (StringUtils.isBlank(executeTemplateQuery.getResultExcelName())) {
            originalFilename = excelInfo.getTemplateOriginalFilename();
        } else {
            originalFilename = executeTemplateQuery.getResultExcelName() + "." + extName;
        }
        fileEntity.setContentType(MediaType.ALL_VALUE)
                .setOriginalFilename(originalFilename)
                .setFileSize(2222L);
        fileEntity.setFileType(fileProperties.getType());
        String nowDate = DateUtil.format(new Date(), "yyyyMMddHHmmss");
        fileEntity.setFileName(nowDate + "." + extName);
        File dest = new File(localPath + File.separator + fileEntity.getFileName());
        fileEntity.setFilePath(localPath + File.separator + fileEntity.getFileName());
        FileOutputStream fout = null;
        try {
            fout = new FileOutputStream(dest);
            excelInfo.getTemplateWorkbook().write(fout);
            fileService.save(fileEntity);
            gmhFileLogService.recordLog(fileEntity, GmhFileLog.OperateType.TEMPL_EXP);
        } finally {
            assert fout != null;
            fout.close();
//            excelInfo.getTemplateWorkbook().close();
        }
        return R.ok();

        // 输出Excel文件 下载出去
//        OutputStream output = response.getOutputStream();
//        response.reset();
//        // 设置文件头
//        response.setHeader("Content-Disposition",
//                "attchement;filename=" + new String((sheetName + ".xls").getBytes("gb2312"), "ISO8859-1"));
//        response.setContentType("application/msexcel");
//        excelInfo.getTemplateWorkbook().write(fout);
//        excelInfo.getTemplateWorkbook().close();

    }

    @GetMapping("/gotoDownload")
    public R gotoDownload() {
        ExcelInfo excelInfo = excelInfoMap.get(request.getSession().getId());
        try {
            excelInfo.getTemplateWorkbook().close();
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            excelInfoMap.remove(request.getSession().getId());
        }
        return R.ok();
    }

    private boolean isChinaName(String val) {
        QueryWrapper<SysBaijiaxing> queryWrapper = new QueryWrapper<>();
        queryWrapper.apply("{0} LIKE CONCAT(xingshi,'%')", val);
        List<SysBaijiaxing> list = baijiaxingService.list(queryWrapper);
        return !list.isEmpty();
    }

    public void oleCode() {
//        Integer nameCellIndex = null;
//        // Sheet:nameList
//        List<ReadNameVo> readNameList = new ArrayList<>();
//        for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
//            List<GmhUser> nameList = new ArrayList<>();
//            Sheet sheet = workbook.getSheetAt(i);
//            for (int r = 0; r <= sheet.getLastRowNum(); r++) {
//                Row row = sheet.getRow(r);
//                if (row == null) {
//                    continue;
//                }
//                List<List<String>> cellList = new ArrayList<>(row.getLastCellNum());
//                for (int c = 0; c < row.getLastCellNum(); c++) {
//                    Cell cell = row.getCell(c);
//                    if (cell == null) {
//                        continue;
//                    }
//                    String colString = CellReference.convertNumToColString(cell.getColumnIndex());
//                    System.out.println(colString);
//
//
//                    String stringCellValue = POIExcelUtil.getStringCellValExcludeBlank(cell);
//                    if (r == 0) {
//                        // 第一行有姓名列
//                        if ("姓名".equals(stringCellValue)) {
//                            nameCellIndex = c;
//                            break;
//                        } else {
//                            // 第一行没有指明姓名列
//                        }
//                    }
//                }
//                if (nameCellIndex != null) {
//                    Cell cell = row.getCell(nameCellIndex);
//                    String stringCellValue = POIExcelUtil.getStringCellValExcludeBlank(cell);
//                    if (!"姓名".equals(stringCellValue) && StrUtil.isNotBlank(stringCellValue)) {
//                        GmhUser user = new GmhUser();
//                        user.setName(stringCellValue);
//                        nameList.add(user);
//                    }
//                } else {
//                    System.out.println("未找到姓名列");
//                }
//            }
//            if (!nameList.isEmpty()) {
//                ReadNameVo readNameVo = new ReadNameVo();
//                readNameVo.setSheetName(sheet.getSheetName());
//                readNameVo.setNameList(nameList);
//                readNameList.add(readNameVo);
//            }
//        }
//        return R.ok().setData(readNameList);
    }

    /**
     * 数据源导入模式枚举
     */
    @Getter
    @AllArgsConstructor
    public enum DataSourceImportModel {

        AUTO_NAME("0", "自动识别姓名"),
        ALL("1", "全部导入");

        private final String value;
        private final String remark;

        public static DataSourceImportModel findEnum(String value) {
            for (DataSourceImportModel item : values()) {
                if (value.equals(item.getValue())) {
                    return item;
                }
            }
            throw new RuntimeException("没有找到对应的枚举");
        }
    }

}
