package com.gmh.utils;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.poi.excel.ExcelFileUtil;
import com.gmh.entity.RowCellNum;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellReference;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * POI Excel工具类
 *
 * @author Chenda
 */
public class POIExcelUtil {

    public static void main(String[] args) throws IOException {
        File file = new File("C:\\Users\\xxche\\Desktop\\20221106_test_excel\\附件3.继续教育证明.xls");
        Workbook workbook = batchCloneSheet(file, 0, 50);
        try {
            FileOutputStream out = new FileOutputStream("C:\\Users\\xxche\\Desktop\\20221106_test_excel\\附件3.继续教育证明222.xls");
            out.flush();
            workbook.write(out);
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 克隆一个Sheet
     *
     * @param workbook     workbook
     * @param srcSheetName 原始sheet页名字
     * @param newSheetName 新sheet页名字
     */
    public static void cloneSheet(Workbook workbook, String srcSheetName, String newSheetName) {
        int index = workbook.getSheetIndex(srcSheetName);
        cloneSheet(workbook, index, newSheetName);
    }

    /**
     * 克隆一个Sheet
     *
     * @param workbook     workbook
     * @param index        原始sheet页索引
     * @param newSheetName 新sheet页名字
     */
    public static void cloneSheet(Workbook workbook, Integer index, String newSheetName) {
        //克隆一个新的sheet
        Sheet newSheet = workbook.cloneSheet(index);
        int sheetIndex = workbook.getSheetIndex(newSheet);
        workbook.setSheetName(sheetIndex, newSheetName);
    }

    /**
     * 克隆一个Sheet
     *
     * @param excelFile    excel文件
     * @param index        原始sheet页索引
     * @param newSheetName 新sheet页名字
     * @throws IOException exception
     */
    public static void cloneSheet(File excelFile, Integer index, String newSheetName) throws IOException {
        Workbook workbook = readExcelFromFile(excelFile);
        cloneSheet(workbook, index, newSheetName);
    }

    /**
     * 复制多个sheet
     * 复制出来的新sheet会按照自然排序数字进行命名，如：1 2 3 4 ...
     *
     * @param workbook workbook
     * @param index    起始sheet
     * @param total    想要复制多少（复制后会删除原始sheet，此参数只需指定总数即可）
     */
    public static Workbook batchCloneSheet(Workbook workbook, Integer index, Integer total) {
        // 将原始sheet重新命名，防止复制出来的新sheet与它重名
        workbook.setSheetName(index, RandomUtil.randomString(5));
        for (int i = 0; i < total; i++) {
            cloneSheet(workbook, index, String.valueOf(i + 1));
        }
        workbook.removeSheetAt(index);
        return workbook;
    }

    /**
     * 复制多个sheet
     * 复制出来的新sheet会按照自然排序数字进行命名，如：1 2 3 4 ...
     *
     * @param excelFile excel文件
     * @param index     起始sheet
     * @param total     想要复制多少（复制后会删除原始sheet，此参数只需指定总数即可）
     */
    public static Workbook batchCloneSheet(File excelFile, Integer index, Integer total) throws IOException {
        Workbook workbook = readExcelFromFile(excelFile);
        return batchCloneSheet(workbook, index, total);
    }

    /**
     * 从文件中读取excel
     *
     * @param file excel文件
     * @return Workbook
     * @throws IOException exception
     */
    public static Workbook readExcelFromFile(File file) throws IOException {
        if (file == null) {
            throw new RuntimeException("文件为空");
        }
        return readExcelFromInputStream(Files.newInputStream(file.toPath()), file.getName());
    }

    /**
     * 从流中读取excel
     *
     * @param inputStream inputStream
     * @return Workbook
     * @throws IOException exception
     */
    @Deprecated
    public static Workbook readExcelFromInputStream(InputStream inputStream) throws IOException {
        return readExcelFromInputStream(inputStream, null);
    }

    /**
     * 从流中读取excel
     *
     * @param inputStream inputStream
     * @param fileName    文件名
     * @return Workbook
     * @throws IOException exception
     */
    public static Workbook readExcelFromInputStream(InputStream inputStream, String fileName) throws IOException {
        if (StrUtil.isNotBlank(fileName)) {
            String[] split = fileName.split("\\.");
            if (split.length == 0 || split.length == 1) {
                throw new RuntimeException("文件格式错误");
            }
            String suffix = split[split.length - 1];
            if ("xls".equals(suffix)) {
                return new HSSFWorkbook(inputStream);
            } else if ("xlsx".equals(suffix)) {
                return new XSSFWorkbook(inputStream);
            } else {
                throw new RuntimeException("文件格式错误");
            }
        }
        if (ExcelFileUtil.isXls(inputStream)) {
            return new HSSFWorkbook(inputStream);
        } else if (ExcelFileUtil.isXlsx(inputStream)) {
            return new XSSFWorkbook(inputStream);
        } else {
            throw new RuntimeException("文件格式错误");
        }
    }

    /**
     * 从excel单元格中获取字符串类型的值，会排除字符串中的所有空格
     * 例：
     * - "张  叁" -> "张叁"
     * - "麻花   " -> "麻花"
     * - " 瑶 海  " -> "瑶海"
     *
     * @param cell cell
     * @return cellValue
     */
    public static String getStringCellValExcludeBlank(Cell cell) {
        return getStringCellVal(cell).replaceAll("\\s*", "");
    }

    /**
     * 从excel单元格中获取字符串类型的值
     *
     * @param cell cell
     * @return cellValue
     */
    public static String getStringCellVal(Cell cell) {
        DataFormatter dataFormatter = new DataFormatter();
        return dataFormatter.formatCellValue(cell);
    }

    /**
     * Excel 转换为List<Map<String, String>>对象
     *
     * @param workbook   workbook
     * @param sheetIndex 当前sheet页索引
     * @return List<Map < String, String>>
     */
    public static List<Map<String, String>> toListMap(Workbook workbook, int sheetIndex) {
        return toListMap(workbook, sheetIndex, -1);
    }

    /**
     * Excel 转换为List<Map<String, String>>对象
     *
     * @param workbook    workbook
     * @param sheetIndex  当前sheet页索引
     * @param titleRowNum 标题行索引，索引值<0则代表没有标题行
     * @return List<Map < String, String>>
     */
    public static List<Map<String, String>> toListMap(Workbook workbook, int sheetIndex, int titleRowNum) {
        return toListMap(workbook.getSheetAt(sheetIndex), titleRowNum);
    }

    /**
     * Excel 转换为List<Map<String, String>>对象，没有title行的
     *
     * @param sheet workbook
     * @return List<Map < String, String>>
     */
    public static List<Map<String, String>> toListMap(Sheet sheet) {
        return toListMap(sheet, -1);
    }

    /**
     * Excel 转换为List<Map<String, String>>对象
     *
     * @param sheet       当前sheet页对象
     * @param titleRowNum 标题行索引，索引值<0则代表没有标题行
     * @return List<Map < String, String>>
     */
    public static List<Map<String, String>> toListMap(Sheet sheet, int titleRowNum) {
        List<Map<String, String>> list = new ArrayList<>();
        List<String> mapKeys = new ArrayList<>();
        if (titleRowNum >= 0) {
            Row titleRow = sheet.getRow(titleRowNum);
            if (titleRow == null) {
                return list;
            }
            for (int i = 0; i < titleRow.getLastCellNum(); i++) {
                Cell cell = titleRow.getCell(i);
                if (cell == null) {
                    break;
                }
                String key = getStringCellValExcludeBlank(cell);
                mapKeys.add(key);
            }
        } else {
            // 根据第一行列的数量去找下面所有的列
            Row titleRow = sheet.getRow(0);
            if (titleRow == null) {
                return list;
            }
            for (int i = 0; i < titleRow.getLastCellNum(); i++) {
                Cell cell = titleRow.getCell(i);
                if (cell == null) {
                    break;
                }
                String colString = CellReference.convertNumToColString(cell.getColumnIndex());
                mapKeys.add(colString);
            }
        }

        for (int i = 0; i <= sheet.getLastRowNum(); i++) {
            // 指定了标题列的情况下获取数据时跳过标题列
            if (titleRowNum >= 0 && i == titleRowNum) {
                continue;
            }
            Row row = sheet.getRow(i);
            if (row == null) {
                continue;
            }
            Map<String, String> map = new LinkedHashMap<>();
            for (int c = 0; c < mapKeys.size(); c++) {
                map.put(mapKeys.get(c), getStringCellValExcludeBlank(row.getCell(c)));
            }
            list.add(map);
        }
        return list;
    }

    /**
     * 删除模板表格(除了 noDelSheet之外所有的)
     */
    public static void removeModelSheet(Workbook wb, Sheet noDelSheet) {
        int sheetIndex = wb.getSheetIndex(noDelSheet);
        removeModelSheet(wb, sheetIndex);
    }

    /**
     * 删除模板表格(除了 name之外所有的)
     */
    public static void removeModelSheet(Workbook wb, String name) {
        int sheetIndex = wb.getSheetIndex(wb.getSheet(name));
        removeModelSheet(wb, sheetIndex);
    }

    /**
     * 删除模板表格(除了 id之外所有的)
     */
    public static void removeModelSheet(Workbook wb, int id) {
        int numberOfSheets = wb.getNumberOfSheets();
        for (int i = numberOfSheets - 1; i > -1; i--) {
            if (i != id) {
                wb.removeSheetAt(i);
            }
        }
        //设置默认显示第一页
        wb.setActiveSheet(0);
    }

    /**
     * 将单元格逻辑坐标转换为物理坐标
     *
     * @param cellRowStr 字符串形式的单元格坐标 如：A3
     * @return
     */
    public static RowCellNum analysisCellRowNum(String cellRowStr) {
        return analysisCellRowNum(new RowCellNum().setCellRowStr(cellRowStr));
    }

    /**
     * 将单元格逻辑坐标转换为物理坐标
     *
     * @param rowCellNum
     * @return
     */
    public static RowCellNum analysisCellRowNum(RowCellNum rowCellNum) {
        String cellStrNum = rowCellNum.getCellRowStr().replaceAll("[^a-z^A-Z]", "");
        String rowNum = rowCellNum.getCellRowStr().replaceAll("[^0-9]", "");
        int cellIndex = CellReference.convertColStringToIndex(cellStrNum);
        return rowCellNum.setCellNum(cellIndex).setRowNum(Integer.parseInt(rowNum) - 1);
    }
}